<?php

namespace Drupal\pagetree\Service;

use Drupal\Core\Entity\ContentEntityBase;

class StateChange
{

    /*
     * Publish a page and all rows, components and contents within.
     *
     * Returns a tree array of all published node and revision ids.
     *
     * @return The published node and revision ids.
     */
    public function publish(ContentEntityBase &$entity, $message)
    {
        $handlers = \Drupal::service('plugin.manager.pagetree_handler')->getHandlers();
        foreach ($handlers as $handler) {
            $handler->publish($entity, $message);
        }
    }

    /*
     * Publish a page and all rows, components and contents within.
     *
     * Returns a tree array of all published node and revision ids.
     *
     * @return The published node and revision ids.
     */
    public function unpublish(ContentEntityBase &$entity, $message)
    {
        $handlers = \Drupal::service('plugin.manager.pagetree_handler')->getHandlers();
        foreach ($handlers as $handler) {
            $handler->unpublish($entity, $message);
        }
    }

    /**
     *
     * Clones a entity and passes each language version to the registered handlers.
     * Returns the cloned entity.
     *
     * @param ContentEntityBase $entity The entity to clone.
     * @return ContentEntityBase The cloned entity in the default translation.
     */
    public function copy(ContentEntityBase &$entity)
    {
        $clone = null;
        $handlers = \Drupal::service('plugin.manager.pagetree_handler')->getHandlers();
        foreach ($handlers as $handler) {
            if ($clone != null) {
                $clone = $clone->getUntranslated();
                $entity = $entity->getUntranslated();
                $clone = $handler->copy($entity, $clone);
                foreach ($clone->getTranslationLanguages(false) as $language) {
                    $clone = $clone->getTranslation($language->getId());
                    $entity = $entity->getTranslation($language->getId());
                    $clone = $handler->copy($entity, $clone);
                }
            } else {
                $clone = $handler->copy($entity, $clone);
            }
        }
        foreach ($clone->getTranslationLanguages(true) as $language) {
            $clone->getTranslation($language->getId())->save();
        }
        $clone = $clone->getUntranslated();
        return $clone;
    }
}
